# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .dvir_2_device_object_response_body import Dvir2DeviceObjectResponseBody
from .dvir_2_related_device_object_response_body_asset_type import Dvir2RelatedDeviceObjectResponseBodyAssetType


class Dvir2RelatedDeviceObjectResponseBody(UniversalBaseModel):
    """
    A device (vehicle or trailer) related to the DVIR.
    """

    asset_type: typing_extensions.Annotated[
        Dvir2RelatedDeviceObjectResponseBodyAssetType,
        FieldMetadata(alias="assetType"),
        pydantic.Field(alias="assetType", description="The type of asset.  Valid values: `Vehicle`, `Trailer`"),
    ]
    device: typing.Optional[Dvir2DeviceObjectResponseBody] = None
    device_id: typing_extensions.Annotated[
        int, FieldMetadata(alias="deviceId"), pydantic.Field(alias="deviceId", description="The ID of the device.")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ambient_temperature_details_object_request_body import AmbientTemperatureDetailsObjectRequestBody
from .cell_signal_loss_details_object_request_body import CellSignalLossDetailsObjectRequestBody
from .def_level_trigger_details_object_request_body import DefLevelTriggerDetailsObjectRequestBody
from .device_movement_trigger_details_object_request_body import DeviceMovementTriggerDetailsObjectRequestBody
from .driver_document_submitted_details_object_request_body import DriverDocumentSubmittedDetailsObjectRequestBody
from .dvir_submitted_device_trigger_details_object_request_body import (
    DvirSubmittedDeviceTriggerDetailsObjectRequestBody,
)
from .engine_idle_trigger_details_object_request_body import EngineIdleTriggerDetailsObjectRequestBody
from .engine_off_details_object_request_body import EngineOffDetailsObjectRequestBody
from .engine_on_details_object_request_body import EngineOnDetailsObjectRequestBody
from .fuel_level_trigger_details_object_request_body import FuelLevelTriggerDetailsObjectRequestBody
from .gateway_disconnected_details_object_request_body import GatewayDisconnectedDetailsObjectRequestBody
from .gateway_unplugged_trigger_details_object_request_body import GatewayUnpluggedTriggerDetailsObjectRequestBody
from .geofence_entry_trigger_details_object_request_body import GeofenceEntryTriggerDetailsObjectRequestBody
from .geofence_exit_trigger_details_object_request_body import GeofenceExitTriggerDetailsObjectRequestBody
from .gps_signal_loss_details_object_request_body import GpsSignalLossDetailsObjectRequestBody
from .harsh_event_trigger_details_object_request_body import HarshEventTriggerDetailsObjectRequestBody
from .hos_violation_trigger_details_object_request_body import HosViolationTriggerDetailsObjectRequestBody
from .inside_geofence_trigger_details_object_request_body import InsideGeofenceTriggerDetailsObjectRequestBody
from .out_of_route_details_object_request_body import OutOfRouteDetailsObjectRequestBody
from .outside_geofence_trigger_details_object_request_body import OutsideGeofenceTriggerDetailsObjectRequestBody
from .panic_button_details_object_request_body import PanicButtonDetailsObjectRequestBody
from .reading_trigger_details_object_request_body import ReadingTriggerDetailsObjectRequestBody
from .route_stop_estimated_arrival_details_object_request_body import RouteStopEstimatedArrivalDetailsObjectRequestBody
from .safety_behavior_trigger_details_object_request_body import SafetyBehaviorTriggerDetailsObjectRequestBody
from .scheduled_maintenance_by_engine_hours_details_object_request_body import (
    ScheduledMaintenanceByEngineHoursDetailsObjectRequestBody,
)
from .scheduled_maintenance_odometer_trigger_details_object_request_body import (
    ScheduledMaintenanceOdometerTriggerDetailsObjectRequestBody,
)
from .scheduled_maintenance_trigger_details_object_request_body import (
    ScheduledMaintenanceTriggerDetailsObjectRequestBody,
)
from .speed_trigger_details_object_request_body import SpeedTriggerDetailsObjectRequestBody
from .sudden_fuel_level_drop_trigger_details_object_request_body import (
    SuddenFuelLevelDropTriggerDetailsObjectRequestBody,
)
from .sudden_fuel_level_rise_trigger_details_object_request_body import (
    SuddenFuelLevelRiseTriggerDetailsObjectRequestBody,
)
from .tire_fault_code_details_object_request_body import TireFaultCodeDetailsObjectRequestBody
from .training_assignment_near_due_date_trigger_details_object_request_body import (
    TrainingAssignmentNearDueDateTriggerDetailsObjectRequestBody,
)
from .unassigned_driving_trigger_details_object_request_body import UnassignedDrivingTriggerDetailsObjectRequestBody
from .vehicle_batter_voltage_details_object_request_body import VehicleBatterVoltageDetailsObjectRequestBody
from .vehicle_fault_code_details_object_request_body import VehicleFaultCodeDetailsObjectRequestBody


class TriggerParamsObjectRequestBody(UniversalBaseModel):
    """
    The trigger type specific details. Only the field that corresponds to the trigger type is filled in.
    """

    ambient_temperature: typing_extensions.Annotated[
        typing.Optional[AmbientTemperatureDetailsObjectRequestBody],
        FieldMetadata(alias="ambientTemperature"),
        pydantic.Field(alias="ambientTemperature"),
    ] = None
    cell_signal_loss: typing_extensions.Annotated[
        typing.Optional[CellSignalLossDetailsObjectRequestBody],
        FieldMetadata(alias="cellSignalLoss"),
        pydantic.Field(alias="cellSignalLoss"),
    ] = None
    def_level: typing_extensions.Annotated[
        typing.Optional[DefLevelTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="defLevel"),
        pydantic.Field(alias="defLevel"),
    ] = None
    device_movement: typing_extensions.Annotated[
        typing.Optional[DeviceMovementTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="deviceMovement"),
        pydantic.Field(alias="deviceMovement"),
    ] = None
    document_submitted: typing_extensions.Annotated[
        typing.Optional[DriverDocumentSubmittedDetailsObjectRequestBody],
        FieldMetadata(alias="documentSubmitted"),
        pydantic.Field(alias="documentSubmitted"),
    ] = None
    dvir_submitted_device: typing_extensions.Annotated[
        typing.Optional[DvirSubmittedDeviceTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="dvirSubmittedDevice"),
        pydantic.Field(alias="dvirSubmittedDevice"),
    ] = None
    engine_idle: typing_extensions.Annotated[
        typing.Optional[EngineIdleTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="engineIdle"),
        pydantic.Field(alias="engineIdle"),
    ] = None
    engine_off: typing_extensions.Annotated[
        typing.Optional[EngineOffDetailsObjectRequestBody],
        FieldMetadata(alias="engineOff"),
        pydantic.Field(alias="engineOff"),
    ] = None
    engine_on: typing_extensions.Annotated[
        typing.Optional[EngineOnDetailsObjectRequestBody],
        FieldMetadata(alias="engineOn"),
        pydantic.Field(alias="engineOn"),
    ] = None
    fuel_level: typing_extensions.Annotated[
        typing.Optional[FuelLevelTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="fuelLevel"),
        pydantic.Field(alias="fuelLevel"),
    ] = None
    gateway_disconnected: typing_extensions.Annotated[
        typing.Optional[GatewayDisconnectedDetailsObjectRequestBody],
        FieldMetadata(alias="gatewayDisconnected"),
        pydantic.Field(alias="gatewayDisconnected"),
    ] = None
    gateway_unplugged: typing_extensions.Annotated[
        typing.Optional[GatewayUnpluggedTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="gatewayUnplugged"),
        pydantic.Field(alias="gatewayUnplugged"),
    ] = None
    geofence_entry: typing_extensions.Annotated[
        typing.Optional[GeofenceEntryTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="geofenceEntry"),
        pydantic.Field(alias="geofenceEntry"),
    ] = None
    geofence_exit: typing_extensions.Annotated[
        typing.Optional[GeofenceExitTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="geofenceExit"),
        pydantic.Field(alias="geofenceExit"),
    ] = None
    gps_signal_loss: typing_extensions.Annotated[
        typing.Optional[GpsSignalLossDetailsObjectRequestBody],
        FieldMetadata(alias="gpsSignalLoss"),
        pydantic.Field(alias="gpsSignalLoss"),
    ] = None
    harsh_event: typing_extensions.Annotated[
        typing.Optional[HarshEventTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="harshEvent"),
        pydantic.Field(alias="harshEvent"),
    ] = None
    hos_violation: typing_extensions.Annotated[
        typing.Optional[HosViolationTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="hosViolation"),
        pydantic.Field(alias="hosViolation"),
    ] = None
    inside_geofence: typing_extensions.Annotated[
        typing.Optional[InsideGeofenceTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="insideGeofence"),
        pydantic.Field(alias="insideGeofence"),
    ] = None
    out_of_route: typing_extensions.Annotated[
        typing.Optional[OutOfRouteDetailsObjectRequestBody],
        FieldMetadata(alias="outOfRoute"),
        pydantic.Field(alias="outOfRoute"),
    ] = None
    outside_geofence: typing_extensions.Annotated[
        typing.Optional[OutsideGeofenceTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="outsideGeofence"),
        pydantic.Field(alias="outsideGeofence"),
    ] = None
    panic_button: typing_extensions.Annotated[
        typing.Optional[PanicButtonDetailsObjectRequestBody],
        FieldMetadata(alias="panicButton"),
        pydantic.Field(alias="panicButton"),
    ] = None
    reading: typing.Optional[ReadingTriggerDetailsObjectRequestBody] = None
    route_stop_estimated_arrival: typing_extensions.Annotated[
        typing.Optional[RouteStopEstimatedArrivalDetailsObjectRequestBody],
        FieldMetadata(alias="routeStopEstimatedArrival"),
        pydantic.Field(alias="routeStopEstimatedArrival"),
    ] = None
    safety_behavior: typing_extensions.Annotated[
        typing.Optional[SafetyBehaviorTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="safetyBehavior"),
        pydantic.Field(alias="safetyBehavior"),
    ] = None
    scheduled_maintenance: typing_extensions.Annotated[
        typing.Optional[ScheduledMaintenanceTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="scheduledMaintenance"),
        pydantic.Field(alias="scheduledMaintenance"),
    ] = None
    scheduled_maintenance_by_engine_hours: typing_extensions.Annotated[
        typing.Optional[ScheduledMaintenanceByEngineHoursDetailsObjectRequestBody],
        FieldMetadata(alias="scheduledMaintenanceByEngineHours"),
        pydantic.Field(alias="scheduledMaintenanceByEngineHours"),
    ] = None
    scheduled_maintenance_odometer: typing_extensions.Annotated[
        typing.Optional[ScheduledMaintenanceOdometerTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="scheduledMaintenanceOdometer"),
        pydantic.Field(alias="scheduledMaintenanceOdometer"),
    ] = None
    speed: typing.Optional[SpeedTriggerDetailsObjectRequestBody] = None
    sudden_fuel_level_drop: typing_extensions.Annotated[
        typing.Optional[SuddenFuelLevelDropTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="suddenFuelLevelDrop"),
        pydantic.Field(alias="suddenFuelLevelDrop"),
    ] = None
    sudden_fuel_level_rise: typing_extensions.Annotated[
        typing.Optional[SuddenFuelLevelRiseTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="suddenFuelLevelRise"),
        pydantic.Field(alias="suddenFuelLevelRise"),
    ] = None
    tire_fault_code: typing_extensions.Annotated[
        typing.Optional[TireFaultCodeDetailsObjectRequestBody],
        FieldMetadata(alias="tireFaultCode"),
        pydantic.Field(alias="tireFaultCode"),
    ] = None
    training_assignment_near_due_date: typing_extensions.Annotated[
        typing.Optional[TrainingAssignmentNearDueDateTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="trainingAssignmentNearDueDate"),
        pydantic.Field(alias="trainingAssignmentNearDueDate"),
    ] = None
    unassigned_driving: typing_extensions.Annotated[
        typing.Optional[UnassignedDrivingTriggerDetailsObjectRequestBody],
        FieldMetadata(alias="unassignedDriving"),
        pydantic.Field(alias="unassignedDriving"),
    ] = None
    vehicle_battery_voltage: typing_extensions.Annotated[
        typing.Optional[VehicleBatterVoltageDetailsObjectRequestBody],
        FieldMetadata(alias="vehicleBatteryVoltage"),
        pydantic.Field(alias="vehicleBatteryVoltage"),
    ] = None
    vehicle_fault_code: typing_extensions.Annotated[
        typing.Optional[VehicleFaultCodeDetailsObjectRequestBody],
        FieldMetadata(alias="vehicleFaultCode"),
        pydantic.Field(alias="vehicleFaultCode"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class HosViolations(UniversalBaseModel):
    """
    Durations the driver has been in violation of HOS rules. See [this page](https://www.samsara.com/fleet/eld-compliance/hours-of-service) for more information on HOS rules.
    """

    cycle_violation_duration_ms: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="cycleViolationDurationMs"),
        pydantic.Field(
            alias="cycleViolationDurationMs",
            description="Time since the driver has surpassed the driving cycle duration limit in milliseconds. For property-carrying drivers, this is the amount of time the driver has been on duty or driving past the 60/70-hour limit in 7/8 days.",
        ),
    ] = None
    shift_driving_violation_duration_ms: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="shiftDrivingViolationDurationMs"),
        pydantic.Field(
            alias="shiftDrivingViolationDurationMs",
            description="Time since the driver has surpassed the driving shift duration limit in milliseconds. For property-carrying drivers, this is the amount of time the driver has been driving past the 11-hour limit.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AmbientTemperatureDetailsObjectRequestBodyOperation = typing.Union[
    typing.Literal["GREATER", "INSIDE_RANGE", "LESS", "OUTSIDE_RANGE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .function_config_response_body_params import FunctionConfigResponseBodyParams
from .function_config_response_body_secrets import FunctionConfigResponseBodySecrets
from .function_schedule_response_body import FunctionScheduleResponseBody


class FunctionConfigResponseBody(UniversalBaseModel):
    """
    Configuration of the Function.
    """

    handler: str = pydantic.Field()
    """
    The handler entrypoint for the function code.
    """

    is_schedule_enabled: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isScheduleEnabled"),
        pydantic.Field(alias="isScheduleEnabled", description="Whether the run schedule is enabled."),
    ]
    params: FunctionConfigResponseBodyParams = pydantic.Field()
    """
    Default parameters for the function.
    """

    schedule: typing.Optional[FunctionScheduleResponseBody] = None
    secrets: FunctionConfigResponseBodySecrets = pydantic.Field()
    """
    Secret names available to the function. Values are always empty.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VendorObjectResponseBody(UniversalBaseModel):
    """
    A maintenance vendor.
    """

    address_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="addressId"),
        pydantic.Field(
            alias="addressId",
            description="Linked Samsara Address ID. Use the Addresses API to retrieve name, address, coordinates, and notes.",
        ),
    ] = None
    category_ids: typing_extensions.Annotated[
        typing.List[str],
        FieldMetadata(alias="categoryIds"),
        pydantic.Field(
            alias="categoryIds",
            description="Category UUIDs for this vendor. Use the Vendor Categories endpoint to resolve names.",
        ),
    ]
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: str = pydantic.Field()
    """
    Unique UUID of the vendor.
    """

    services_provided: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="servicesProvided"),
        pydantic.Field(alias="servicesProvided", description="Description of services provided by the vendor."),
    ] = None
    vendor_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="vendorId"),
        pydantic.Field(
            alias="vendorId",
            description="The vendor's legacy vendor ID from the source system. Multiple vendor locations may share the same vendorId if they belong to the same parent company.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .work_order_money_object_response_body import WorkOrderMoneyObjectResponseBody


class ServiceTaskDefinitionObjectResponseBody(UniversalBaseModel):
    """
    A service task definition object.
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The category of the service task.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A description of the service task.
    """

    estimated_labor_time_minutes: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="estimatedLaborTimeMinutes"),
        pydantic.Field(
            alias="estimatedLaborTimeMinutes", description="Estimated time in minutes to complete the task."
        ),
    ] = None
    estimated_parts_cost: typing_extensions.Annotated[
        typing.Optional[WorkOrderMoneyObjectResponseBody],
        FieldMetadata(alias="estimatedPartsCost"),
        pydantic.Field(alias="estimatedPartsCost"),
    ] = None
    id: str = pydantic.Field()
    """
    The unique id of the service task definition.
    """

    name: str = pydantic.Field()
    """
    The name of the service task.
    """

    subcategory: typing.Optional[str] = pydantic.Field(default=None)
    """
    The subcategory of the service task.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.hos_clocks_response import HosClocksResponse
from ..types.hos_daily_logs_get_hos_daily_logs_response_body import HosDailyLogsGetHosDailyLogsResponseBody
from ..types.hos_logs_response import HosLogsResponse
from ..types.hos_violations_get_hos_violations_response_body import HosViolationsGetHosViolationsResponseBody
from ..types.v_1_hos_authentication_logs_response import V1HosAuthenticationLogsResponse
from .types.get_hos_daily_logs_request_driver_activation_status import GetHosDailyLogsRequestDriverActivationStatus
from .types.get_hos_daily_logs_request_expand import GetHosDailyLogsRequestExpand

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawHoursOfServiceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_hos_clocks(
        self,
        *,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HosClocksResponse]:
        """
        Get the current HOS status for all drivers. Note that this includes inactive as well as active drivers. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getFleetHosLogsSummary).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs. Example: `driverIds=1234,5678`

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HosClocksResponse]
            List of current HOS clock information for the specified drivers.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/hos/clocks",
            method="GET",
            params={
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "driverIds": driver_ids,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HosClocksResponse,
                    parse_obj_as(
                        type_=HosClocksResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_hos_daily_logs(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        start_date: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        driver_activation_status: typing.Optional[GetHosDailyLogsRequestDriverActivationStatus] = None,
        after: typing.Optional[str] = None,
        expand: typing.Optional[GetHosDailyLogsRequestExpand] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HosDailyLogsGetHosDailyLogsResponseBody]:
        """
        Get summarized daily Hours of Service charts for the specified drivers.

        The time range for a log is defined by the `driver`'s `eldDayStartHour`. This value is configurable per driver.

        The `startDate` and `endDate` parameters indicate the date range you'd like to retrieve daily logs for. A daily log will be returned if its `startTime` is on any of the days within in this date range (inclusive of `startDate` and `endDate`).

        **Note:** If data is still being uploaded from the Samsara Driver App, it may not be completely reflected in the response from this endpoint. The best practice is to wait a couple of days before querying this endpoint to make sure that all data from the Samsara Driver App has been uploaded.

        If you are using the legacy version of this endpoint and looking for its documentation, you can find it [here](https://www.samsara.com/api-legacy#operation/getFleetDriversHosDailyLogs).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        start_date : typing.Optional[str]
             A start date in YYYY-MM-DD. This is a date only without an associated time. Example: `2019-06-13`. This is a required field

        end_date : typing.Optional[str]
             An end date in YYYY-MM-DD. This is a date only without an associated time. Must be greater than or equal to the start date. Example: `2019-07-21`. This is a required field

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        driver_activation_status : typing.Optional[GetHosDailyLogsRequestDriverActivationStatus]
            If value is `deactivated`, only drivers that are deactivated will appear in the response. This parameter will default to `active` if not provided (fetching only active drivers).  Valid values: `active`, `deactivated`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        expand : typing.Optional[GetHosDailyLogsRequestExpand]
            Expands the specified value(s) in the response object. Expansion populates additional fields in an object, if supported. Unsupported fields are ignored. To expand multiple fields, input a comma-separated list.

            Valid value: `vehicle`  Valid values: `vehicle`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HosDailyLogsGetHosDailyLogsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/hos/daily-logs",
            method="GET",
            params={
                "driverIds": driver_ids,
                "startDate": start_date,
                "endDate": end_date,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "driverActivationStatus": driver_activation_status,
                "after": after,
                "expand": expand,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HosDailyLogsGetHosDailyLogsResponseBody,
                    parse_obj_as(
                        type_=HosDailyLogsGetHosDailyLogsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_hos_logs(
        self,
        *,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HosLogsResponse]:
        """
        Returns HOS logs between a given `startTime` and `endTime`. The logs can be further filtered using tags or by providing a list of driver IDs (including external IDs). The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getFleetHosLogs).

        **Note:** If data is still being uploaded from the Samsara Driver App, it may not be completely reflected in the response from this endpoint. The best practice is to wait a couple of days before querying this endpoint to make sure that all data from the Samsara Driver App has been uploaded.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs. Example: `driverIds=1234,5678`

        start_time : typing.Optional[str]
            A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HosLogsResponse]
            List of the last known HOS log entries for the specified drivers.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/hos/logs",
            method="GET",
            params={
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "driverIds": driver_ids,
                "startTime": start_time,
                "endTime": end_time,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HosLogsResponse,
                    parse_obj_as(
                        type_=HosLogsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_hos_violations(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        types: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HosViolationsGetHosViolationsResponseBody]:
        """
        Get active Hours of Service violations for the specified drivers.

        The day object time range for a violation is defined by the `driver`'s `eldDayStartHour`. This value is configurable per driver.

        The `startTime` and `endTime` parameters indicate the datetime range you'd like to retrieve violations for. A violation will be returned if its `violationStartTime` falls within this datetime range (inclusive of `startTime` and `endTime`)

        **Note:** The following are all the violation types with a short explanation about what each of them means: `californiaMealbreakMissed` (Missed California Meal Break), `cycleHoursOn` (Cycle Limit), `cycleOffHoursAfterOnDutyHours` (Cycle 2 Limit), `dailyDrivingHours` (Daily Driving Limit), `dailyOffDutyDeferralAddToDay2Consecutive` (Daily Off-Duty Deferral: Add To Day2 Consecutive), `dailyOffDutyDeferralNotPartMandatory` (Daily Off-Duty Deferral: Not Part Of Mandatory), `dailyOffDutyDeferralTwoDayDrivingLimit` (Daily Off-Duty Deferral: 2 Day Driving Limit), `dailyOffDutyDeferralTwoDayOffDuty` (Daily Off-Duty Deferral: 2 Day Off Duty), `dailyOffDutyNonResetHours` (Daily Off-Duty Time: Non-Reset), `dailyOffDutyTotalHours` (Daily Off-Duty Time), `dailyOnDutyHours` (Daily On-Duty Limit), `mandatory24HoursOffDuty` (24 Hours of Off Duty required), `restbreakMissed` (Missed Rest Break), `shiftDrivingHours` (Shift Driving Limit), `shiftHours` (Shift Duty Limit), `shiftOnDutyHours` (Shift On-Duty Limit), `unsubmittedLogs` (Missing Driver Certification)

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        types : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on violations data based on the violation type enum. Supported types: `NONE, californiaMealbreakMissed, cycleHoursOn, cycleOffHoursAfterOnDutyHours, dailyDrivingHours, dailyOffDutyDeferralAddToDay2Consecutive, dailyOffDutyDeferralNotPartMandatory, dailyOffDutyDeferralTwoDayDrivingLimit, dailyOffDutyDeferralTwoDayOffDuty, dailyOffDutyNonResetHours, dailyOffDutyTotalHours, dailyOnDutyHours, mandatory24HoursOffDuty, restbreakMissed, shiftDrivingHours, shiftHours, shiftOnDutyHours, unsubmittedLogs`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HosViolationsGetHosViolationsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/hos/violations",
            method="GET",
            params={
                "driverIds": driver_ids,
                "startTime": start_time,
                "endTime": end_time,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "types": types,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HosViolationsGetHosViolationsResponseBody,
                    parse_obj_as(
                        type_=HosViolationsGetHosViolationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set_current_duty_status(
        self,
        driver_id: int,
        *,
        duty_status: str,
        location: typing.Optional[str] = OMIT,
        remark: typing.Optional[str] = OMIT,
        status_change_at_ms: typing.Optional[int] = OMIT,
        vehicle_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Set an individual driver’s current duty status to 'On Duty' or 'Off Duty'.

         To ensure compliance with the ELD Mandate, only  authenticated drivers can make direct duty status changes on their own logbook. Any system external to the Samsara Driver App using this endpoint to trigger duty status changes must ensure that such changes are only triggered directly by the driver in question and that the driver has been properly authenticated. This endpoint should not be used to algorithmically trigger duty status changes nor should it be used by personnel besides the driver to trigger duty status changes on the driver’s behalf. Carriers and their drivers are ultimately responsible for maintaining accurate logs and should confirm that their use of the endpoint is compliant with the ELD Mandate.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write ELD Hours of Service (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : int
            ID of the driver for whom the duty status is being set.

        duty_status : str
            Duty status to set the driver to. The only supported values are 'ON_DUTY' and 'OFF_DUTY'.

        location : typing.Optional[str]
            Location to associate the duty status change with.

        remark : typing.Optional[str]
            Remark to associate the duty status change with.

        status_change_at_ms : typing.Optional[int]
            Timestamp that the duty status will begin at specified in milliseconds UNIX time. Defaults to the current time if left blank. This can only be set to up to 8 hours in the past.

        vehicle_id : typing.Optional[int]
            Vehicle ID to associate the duty status change with.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/fleet/drivers/{jsonable_encoder(driver_id)}/hos/duty_status",
            method="POST",
            json={
                "duty_status": duty_status,
                "location": location,
                "remark": remark,
                "status_change_at_ms": status_change_at_ms,
                "vehicle_id": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_fleet_hos_authentication_logs(
        self, *, driver_id: int, start_ms: int, end_ms: int, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1HosAuthenticationLogsResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get the HOS (hours of service) signin and signout logs for the specified driver. The response includes 4 fields that are now deprecated.

        **Note:** If data is still being uploaded from the Samsara Driver App, it may not be completely reflected in the response from this endpoint. The best practice is to wait a couple of days before querying this endpoint to make sure that all data from the Samsara Driver App has been uploaded.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read ELD Hours of Service (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : int
            Driver ID to query.

        start_ms : int
            Beginning of the time range, specified in milliseconds UNIX time.

        end_ms : int
            End of the time range, specified in milliseconds UNIX time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1HosAuthenticationLogsResponse]
            HOS authentication logs for the specified driver.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/fleet/hos_authentication_logs",
            method="GET",
            params={
                "driverId": driver_id,
                "startMs": start_ms,
                "endMs": end_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1HosAuthenticationLogsResponse,
                    parse_obj_as(
                        type_=V1HosAuthenticationLogsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawHoursOfServiceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_hos_clocks(
        self,
        *,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HosClocksResponse]:
        """
        Get the current HOS status for all drivers. Note that this includes inactive as well as active drivers. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getFleetHosLogsSummary).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs. Example: `driverIds=1234,5678`

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HosClocksResponse]
            List of current HOS clock information for the specified drivers.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/hos/clocks",
            method="GET",
            params={
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "driverIds": driver_ids,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HosClocksResponse,
                    parse_obj_as(
                        type_=HosClocksResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_hos_daily_logs(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        start_date: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        driver_activation_status: typing.Optional[GetHosDailyLogsRequestDriverActivationStatus] = None,
        after: typing.Optional[str] = None,
        expand: typing.Optional[GetHosDailyLogsRequestExpand] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HosDailyLogsGetHosDailyLogsResponseBody]:
        """
        Get summarized daily Hours of Service charts for the specified drivers.

        The time range for a log is defined by the `driver`'s `eldDayStartHour`. This value is configurable per driver.

        The `startDate` and `endDate` parameters indicate the date range you'd like to retrieve daily logs for. A daily log will be returned if its `startTime` is on any of the days within in this date range (inclusive of `startDate` and `endDate`).

        **Note:** If data is still being uploaded from the Samsara Driver App, it may not be completely reflected in the response from this endpoint. The best practice is to wait a couple of days before querying this endpoint to make sure that all data from the Samsara Driver App has been uploaded.

        If you are using the legacy version of this endpoint and looking for its documentation, you can find it [here](https://www.samsara.com/api-legacy#operation/getFleetDriversHosDailyLogs).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        start_date : typing.Optional[str]
             A start date in YYYY-MM-DD. This is a date only without an associated time. Example: `2019-06-13`. This is a required field

        end_date : typing.Optional[str]
             An end date in YYYY-MM-DD. This is a date only without an associated time. Must be greater than or equal to the start date. Example: `2019-07-21`. This is a required field

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        driver_activation_status : typing.Optional[GetHosDailyLogsRequestDriverActivationStatus]
            If value is `deactivated`, only drivers that are deactivated will appear in the response. This parameter will default to `active` if not provided (fetching only active drivers).  Valid values: `active`, `deactivated`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        expand : typing.Optional[GetHosDailyLogsRequestExpand]
            Expands the specified value(s) in the response object. Expansion populates additional fields in an object, if supported. Unsupported fields are ignored. To expand multiple fields, input a comma-separated list.

            Valid value: `vehicle`  Valid values: `vehicle`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HosDailyLogsGetHosDailyLogsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/hos/daily-logs",
            method="GET",
            params={
                "driverIds": driver_ids,
                "startDate": start_date,
                "endDate": end_date,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "driverActivationStatus": driver_activation_status,
                "after": after,
                "expand": expand,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HosDailyLogsGetHosDailyLogsResponseBody,
                    parse_obj_as(
                        type_=HosDailyLogsGetHosDailyLogsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_hos_logs(
        self,
        *,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HosLogsResponse]:
        """
        Returns HOS logs between a given `startTime` and `endTime`. The logs can be further filtered using tags or by providing a list of driver IDs (including external IDs). The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getFleetHosLogs).

        **Note:** If data is still being uploaded from the Samsara Driver App, it may not be completely reflected in the response from this endpoint. The best practice is to wait a couple of days before querying this endpoint to make sure that all data from the Samsara Driver App has been uploaded.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs. Example: `driverIds=1234,5678`

        start_time : typing.Optional[str]
            A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HosLogsResponse]
            List of the last known HOS log entries for the specified drivers.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/hos/logs",
            method="GET",
            params={
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "driverIds": driver_ids,
                "startTime": start_time,
                "endTime": end_time,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HosLogsResponse,
                    parse_obj_as(
                        type_=HosLogsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_hos_violations(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        types: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HosViolationsGetHosViolationsResponseBody]:
        """
        Get active Hours of Service violations for the specified drivers.

        The day object time range for a violation is defined by the `driver`'s `eldDayStartHour`. This value is configurable per driver.

        The `startTime` and `endTime` parameters indicate the datetime range you'd like to retrieve violations for. A violation will be returned if its `violationStartTime` falls within this datetime range (inclusive of `startTime` and `endTime`)

        **Note:** The following are all the violation types with a short explanation about what each of them means: `californiaMealbreakMissed` (Missed California Meal Break), `cycleHoursOn` (Cycle Limit), `cycleOffHoursAfterOnDutyHours` (Cycle 2 Limit), `dailyDrivingHours` (Daily Driving Limit), `dailyOffDutyDeferralAddToDay2Consecutive` (Daily Off-Duty Deferral: Add To Day2 Consecutive), `dailyOffDutyDeferralNotPartMandatory` (Daily Off-Duty Deferral: Not Part Of Mandatory), `dailyOffDutyDeferralTwoDayDrivingLimit` (Daily Off-Duty Deferral: 2 Day Driving Limit), `dailyOffDutyDeferralTwoDayOffDuty` (Daily Off-Duty Deferral: 2 Day Off Duty), `dailyOffDutyNonResetHours` (Daily Off-Duty Time: Non-Reset), `dailyOffDutyTotalHours` (Daily Off-Duty Time), `dailyOnDutyHours` (Daily On-Duty Limit), `mandatory24HoursOffDuty` (24 Hours of Off Duty required), `restbreakMissed` (Missed Rest Break), `shiftDrivingHours` (Shift Driving Limit), `shiftHours` (Shift Duty Limit), `shiftOnDutyHours` (Shift On-Duty Limit), `unsubmittedLogs` (Missing Driver Certification)

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        types : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on violations data based on the violation type enum. Supported types: `NONE, californiaMealbreakMissed, cycleHoursOn, cycleOffHoursAfterOnDutyHours, dailyDrivingHours, dailyOffDutyDeferralAddToDay2Consecutive, dailyOffDutyDeferralNotPartMandatory, dailyOffDutyDeferralTwoDayDrivingLimit, dailyOffDutyDeferralTwoDayOffDuty, dailyOffDutyNonResetHours, dailyOffDutyTotalHours, dailyOnDutyHours, mandatory24HoursOffDuty, restbreakMissed, shiftDrivingHours, shiftHours, shiftOnDutyHours, unsubmittedLogs`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HosViolationsGetHosViolationsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/hos/violations",
            method="GET",
            params={
                "driverIds": driver_ids,
                "startTime": start_time,
                "endTime": end_time,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "types": types,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HosViolationsGetHosViolationsResponseBody,
                    parse_obj_as(
                        type_=HosViolationsGetHosViolationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set_current_duty_status(
        self,
        driver_id: int,
        *,
        duty_status: str,
        location: typing.Optional[str] = OMIT,
        remark: typing.Optional[str] = OMIT,
        status_change_at_ms: typing.Optional[int] = OMIT,
        vehicle_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Set an individual driver’s current duty status to 'On Duty' or 'Off Duty'.

         To ensure compliance with the ELD Mandate, only  authenticated drivers can make direct duty status changes on their own logbook. Any system external to the Samsara Driver App using this endpoint to trigger duty status changes must ensure that such changes are only triggered directly by the driver in question and that the driver has been properly authenticated. This endpoint should not be used to algorithmically trigger duty status changes nor should it be used by personnel besides the driver to trigger duty status changes on the driver’s behalf. Carriers and their drivers are ultimately responsible for maintaining accurate logs and should confirm that their use of the endpoint is compliant with the ELD Mandate.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write ELD Hours of Service (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : int
            ID of the driver for whom the duty status is being set.

        duty_status : str
            Duty status to set the driver to. The only supported values are 'ON_DUTY' and 'OFF_DUTY'.

        location : typing.Optional[str]
            Location to associate the duty status change with.

        remark : typing.Optional[str]
            Remark to associate the duty status change with.

        status_change_at_ms : typing.Optional[int]
            Timestamp that the duty status will begin at specified in milliseconds UNIX time. Defaults to the current time if left blank. This can only be set to up to 8 hours in the past.

        vehicle_id : typing.Optional[int]
            Vehicle ID to associate the duty status change with.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/fleet/drivers/{jsonable_encoder(driver_id)}/hos/duty_status",
            method="POST",
            json={
                "duty_status": duty_status,
                "location": location,
                "remark": remark,
                "status_change_at_ms": status_change_at_ms,
                "vehicle_id": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_fleet_hos_authentication_logs(
        self, *, driver_id: int, start_ms: int, end_ms: int, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1HosAuthenticationLogsResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get the HOS (hours of service) signin and signout logs for the specified driver. The response includes 4 fields that are now deprecated.

        **Note:** If data is still being uploaded from the Samsara Driver App, it may not be completely reflected in the response from this endpoint. The best practice is to wait a couple of days before querying this endpoint to make sure that all data from the Samsara Driver App has been uploaded.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read ELD Hours of Service (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : int
            Driver ID to query.

        start_ms : int
            Beginning of the time range, specified in milliseconds UNIX time.

        end_ms : int
            End of the time range, specified in milliseconds UNIX time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1HosAuthenticationLogsResponse]
            HOS authentication logs for the specified driver.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/fleet/hos_authentication_logs",
            method="GET",
            params={
                "driverId": driver_id,
                "startMs": start_ms,
                "endMs": end_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1HosAuthenticationLogsResponse,
                    parse_obj_as(
                        type_=V1HosAuthenticationLogsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_vision_run_by_camera_response_inspection_results import V1VisionRunByCameraResponseInspectionResults
from .v_1_vision_run_by_camera_response_run_summary import V1VisionRunByCameraResponseRunSummary


class V1VisionRunsByCameraAndProgramResponse(UniversalBaseModel):
    device_id: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="deviceId"), pydantic.Field(alias="deviceId")
    ] = None
    ended_at_ms: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="endedAtMs"), pydantic.Field(alias="endedAtMs")
    ] = None
    program_id: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="programId"), pydantic.Field(alias="programId")
    ] = None
    report_metadata: typing_extensions.Annotated[
        typing.Optional[V1VisionRunByCameraResponseRunSummary],
        FieldMetadata(alias="reportMetadata"),
        pydantic.Field(alias="reportMetadata"),
    ] = None
    results: typing.Optional[typing.List[V1VisionRunByCameraResponseInspectionResults]] = None
    started_at_ms: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="startedAtMs"), pydantic.Field(alias="startedAtMs")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

RouteSettingsRequestBodyRouteStartingCondition = typing.Union[
    typing.Literal["departFirstStop", "arriveFirstStop"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .equipment_engine_rpm import EquipmentEngineRpm
from .equipment_engine_seconds import EquipmentEngineSeconds
from .equipment_engine_state import EquipmentEngineState
from .equipment_engine_total_idle_time_minutes import EquipmentEngineTotalIdleTimeMinutes
from .equipment_fuel_percent import EquipmentFuelPercent
from .equipment_gateway_engine_seconds import EquipmentGatewayEngineSeconds
from .equipment_gateway_engine_state import EquipmentGatewayEngineState
from .equipment_gateway_j_1939_engine_seconds import EquipmentGatewayJ1939EngineSeconds
from .equipment_gps_odometer_meters import EquipmentGpsOdometerMeters
from .equipment_id import EquipmentId
from .equipment_name import EquipmentName
from .equipment_obd_engine_seconds import EquipmentObdEngineSeconds
from .equipment_obd_engine_state import EquipmentObdEngineState
from .equipment_stats_gps import EquipmentStatsGps


class EquipmentStatsListResponseData(UniversalBaseModel):
    """
    A unit of equipment and its time-series of stats events.
    """

    engine_rpm: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentEngineRpm]],
        FieldMetadata(alias="engineRpm"),
        pydantic.Field(
            alias="engineRpm", description="A time-series of engine RPM readings for the given unit of equipment."
        ),
    ] = None
    engine_seconds: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentEngineSeconds]],
        FieldMetadata(alias="engineSeconds"),
        pydantic.Field(
            alias="engineSeconds",
            description="[DEPRECATED] Please use either `gatewayEngineSeconds`, `obdEngineSeconds`, or `gatewayJ1939EngineSeconds`.",
        ),
    ] = None
    engine_states: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentEngineState]],
        FieldMetadata(alias="engineStates"),
        pydantic.Field(
            alias="engineStates",
            description="[DEPRECATED] Please use either `gatewayEngineStates` or `obdEngineStates`.",
        ),
    ] = None
    engine_total_idle_time_minutes: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentEngineTotalIdleTimeMinutes]],
        FieldMetadata(alias="engineTotalIdleTimeMinutes"),
        pydantic.Field(
            alias="engineTotalIdleTimeMinutes", description="A time-series of engine total idle time minutes. (Beta)"
        ),
    ] = None
    fuel_percents: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentFuelPercent]],
        FieldMetadata(alias="fuelPercents"),
        pydantic.Field(
            alias="fuelPercents",
            description="A time-series of fuel percent level changes for the given unit of equipment.",
        ),
    ] = None
    gateway_engine_seconds: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentGatewayEngineSeconds]],
        FieldMetadata(alias="gatewayEngineSeconds"),
        pydantic.Field(
            alias="gatewayEngineSeconds",
            description="A time-series of engine seconds readings for the given unit of equipment as an approximate based on readings from the AG26's aux/digio cable.",
        ),
    ] = None
    gateway_engine_states: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentGatewayEngineState]],
        FieldMetadata(alias="gatewayEngineStates"),
        pydantic.Field(
            alias="gatewayEngineStates",
            description="A time-series of engine state changes (as read from the AG26's aux/digio cable) for the given unit of equipment.",
        ),
    ] = None
    gateway_j_1939_engine_seconds: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentGatewayJ1939EngineSeconds]],
        FieldMetadata(alias="gatewayJ1939EngineSeconds"),
        pydantic.Field(
            alias="gatewayJ1939EngineSeconds",
            description="A time-series of engine seconds readings for the given unit of equipment as an approximate based on readings from the AG26's CAT/J1939 cable.",
        ),
    ] = None
    gps: typing.Optional[typing.List[EquipmentStatsGps]] = pydantic.Field(default=None)
    """
    A time-series of GPS locations.
    """

    gps_odometer_meters: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentGpsOdometerMeters]],
        FieldMetadata(alias="gpsOdometerMeters"),
        pydantic.Field(
            alias="gpsOdometerMeters",
            description="A time-series of GPS odometer readings for the given unit of equipment.",
        ),
    ] = None
    id: EquipmentId
    name: EquipmentName
    obd_engine_seconds: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentObdEngineSeconds]],
        FieldMetadata(alias="obdEngineSeconds"),
        pydantic.Field(
            alias="obdEngineSeconds",
            description="A time-series of engine seconds readings for the given unit of equipment directly from on-board diagnostics.",
        ),
    ] = None
    obd_engine_states: typing_extensions.Annotated[
        typing.Optional[typing.List[EquipmentObdEngineState]],
        FieldMetadata(alias="obdEngineStates"),
        pydantic.Field(
            alias="obdEngineStates",
            description="A time-series of engine state changes (as read from on-board diagnostics) for the given unit of equipment.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

GatewayResponseObjectResponseBodyModel = typing.Union[
    typing.Literal[
        "AG15",
        "AG24",
        "AG24EU",
        "AG26",
        "AG26EU",
        "AG41",
        "AG41EU",
        "AG45",
        "AG45EU",
        "AG46",
        "AG46EU",
        "AG46P",
        "AG46PEU",
        "AG51",
        "AG51EU",
        "AG52",
        "AG52EU",
        "AG53",
        "AG53EU",
        "IG15",
        "IG21",
        "IG41",
        "IG61",
        "SG1",
        "SG1B",
        "SG1G",
        "SG1G32",
        "SG1x",
        "VG32",
        "VG33",
        "VG34",
        "VG34EU",
        "VG34FN",
        "VG34M",
        "VG54ATT",
        "VG54EU",
        "VG54FN",
        "VG54NA",
        "VG54NAE",
        "VG54NAH",
        "VG55EU",
        "VG55FN",
        "VG55NA",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .forms_table_column_object_response_body_type import FormsTableColumnObjectResponseBodyType


class FormsTableColumnObjectResponseBody(UniversalBaseModel):
    """
    Defines a column in a table form input field.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the column.
    """

    label: str = pydantic.Field()
    """
    Label of the column.
    """

    type: FormsTableColumnObjectResponseBodyType = pydantic.Field()
    """
    Type of the column field.  Valid values: `text`, `number`, `datetime`, `check_boxes`, `multiple_choice`, `signature`, `media`, `person`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_stats_fault_codes_j_1939_lights import VehicleStatsFaultCodesJ1939Lights
from .vehicle_stats_fault_codes_j_1939_trouble_codes import VehicleStatsFaultCodesJ1939TroubleCodes


class VehicleStatsFaultCodesJ1939(UniversalBaseModel):
    """
    Vehicle fault codes for J1939 vehicles.
    """

    check_engine_lights: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesJ1939Lights],
        FieldMetadata(alias="checkEngineLights"),
        pydantic.Field(alias="checkEngineLights"),
    ] = None
    diagnostic_trouble_codes: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesJ1939TroubleCodes],
        FieldMetadata(alias="diagnosticTroubleCodes"),
        pydantic.Field(alias="diagnosticTroubleCodes"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FormSubmissionRequestMediaItemObjectRequestBody(UniversalBaseModel):
    """
    A media item in a form submission request.
    """

    base_64_payload: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="base64Payload"),
        pydantic.Field(alias="base64Payload", description="Base64-encoded binary content of the media."),
    ]
    media_type: typing_extensions.Annotated[
        str, FieldMetadata(alias="mediaType"), pydantic.Field(alias="mediaType", description="MIME type of the media.")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class DriverEfficiencyDataObjectResponseBody(UniversalBaseModel):
    """
    Driver Efficiency score data. This object is returned by default or when the “score” format is specified in “dataFormats”.
    """

    anticipation_score: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="anticipationScore"),
        pydantic.Field(
            alias="anticipationScore",
            description="Represents the anticipation score for the driver. The score will be in either number or letter format depending on the organisation config.",
        ),
    ] = None
    coasting_score: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="coastingScore"),
        pydantic.Field(
            alias="coastingScore",
            description="Represents the coasting score for the driver. The score will be in either number or letter format depending on the organisation config.",
        ),
    ] = None
    cruise_control_score: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="cruiseControlScore"),
        pydantic.Field(
            alias="cruiseControlScore",
            description="Represents the cruise control score for the driver. The score will be in either number or letter format depending on the organisation config.",
        ),
    ] = None
    green_band_score: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="greenBandScore"),
        pydantic.Field(
            alias="greenBandScore",
            description="Represents the green band score for the driver. The score will be in either number or letter format depending on the organisation config.",
        ),
    ] = None
    high_torque_score: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="highTorqueScore"),
        pydantic.Field(
            alias="highTorqueScore",
            description="Represents the high torque score for the driver. The score will be in either number or letter format depending on the organisation config.",
        ),
    ] = None
    idling_score: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="idlingScore"),
        pydantic.Field(
            alias="idlingScore",
            description="Represents the idling score for the driver.The score will be in either number or letter format depending on the organisation config.",
        ),
    ] = None
    over_speed_score: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="overSpeedScore"),
        pydantic.Field(
            alias="overSpeedScore",
            description="Represents the over speed score for the driver. The score will be in either number or letter format depending on the organisation config.",
        ),
    ] = None
    overall_score: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="overallScore"),
        pydantic.Field(
            alias="overallScore",
            description="Represents the overall score for the driver. The score will be in either number (0-100) as a string or letter format (A-G) depending on the organisation config.",
        ),
    ]
    wear_free_brake_score: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="wearFreeBrakeScore"),
        pydantic.Field(
            alias="wearFreeBrakeScore",
            description="Represents the ware-free breaking score for the driver. The score will be in either number or letter format depending on the organisation config.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .assets_location_link_response_config_object_response_body import (
    AssetsLocationLinkResponseConfigObjectResponseBody,
)
from .assets_near_location_link_config_object_response_body import AssetsNearLocationLinkConfigObjectResponseBody
from .assets_on_route_link_config_object_response_body import AssetsOnRouteLinkConfigObjectResponseBody
from .live_sharing_link_full_response_object_response_body_type import LiveSharingLinkFullResponseObjectResponseBodyType


class LiveSharingLinkFullResponseObjectResponseBody(UniversalBaseModel):
    """
    Live Sharing Link object
    """

    assets_location_link_config: typing_extensions.Annotated[
        typing.Optional[AssetsLocationLinkResponseConfigObjectResponseBody],
        FieldMetadata(alias="assetsLocationLinkConfig"),
        pydantic.Field(alias="assetsLocationLinkConfig"),
    ] = None
    assets_near_location_link_config: typing_extensions.Annotated[
        typing.Optional[AssetsNearLocationLinkConfigObjectResponseBody],
        FieldMetadata(alias="assetsNearLocationLinkConfig"),
        pydantic.Field(alias="assetsNearLocationLinkConfig"),
    ] = None
    assets_on_route_link_config: typing_extensions.Annotated[
        typing.Optional[AssetsOnRouteLinkConfigObjectResponseBody],
        FieldMetadata(alias="assetsOnRouteLinkConfig"),
        pydantic.Field(alias="assetsOnRouteLinkConfig"),
    ] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description for the Live Sharing Link (not applicable for 'assetsOnRoute' type).
    """

    expires_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="expiresAtTime"),
        pydantic.Field(alias="expiresAtTime", description="Date that this link expires, in RFC 3339 format."),
    ] = None
    id: str = pydantic.Field()
    """
    Unique identifier for the Live Sharing Link.
    """

    live_sharing_url: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="liveSharingUrl"),
        pydantic.Field(alias="liveSharingUrl", description="The shareable URL of the vehicle's location."),
    ]
    name: str = pydantic.Field()
    """
    Name of the Live Sharing Link.
    """

    type: LiveSharingLinkFullResponseObjectResponseBodyType = pydantic.Field()
    """
    Type of the Live Sharing Link.  Valid values: `assetsLocation`, `assetsNearLocation`, `assetsOnRoute`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .address_address_types_item import AddressAddressTypesItem
from .address_external_ids import AddressExternalIds
from .address_geofence import AddressGeofence
from .contact_tiny_response import ContactTinyResponse
from .tag_tiny_response import TagTinyResponse


class Address(UniversalBaseModel):
    """
    An Address object.
    """

    address_types: typing_extensions.Annotated[
        typing.Optional[typing.List[AddressAddressTypesItem]],
        FieldMetadata(alias="addressTypes"),
        pydantic.Field(
            alias="addressTypes",
            description="Reporting location type associated with the address (used for ELD reporting purposes). Valid values: `yard`, `shortHaul`, `workforceSite`, `riskZone`, `industrialSite`, `alertsOnly`, `agricultureSource`, `avoidanceZone`, `knownGPSJammingZone`, `authorizedZone`, `unauthorizedZone`, `vendor`, `inventory`.",
        ),
    ] = None
    contacts: typing.Optional[typing.List[ContactTinyResponse]] = pydantic.Field(default=None)
    """
    An array Contact mini-objects that are associated the Address.
    """

    created_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime", description="The date and time this address was created in RFC 3339 format."
        ),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[AddressExternalIds],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(
            alias="externalIds",
            description="The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.",
        ),
    ] = None
    formatted_address: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="formattedAddress"),
        pydantic.Field(
            alias="formattedAddress",
            description="The full street address for this address/geofence, as it might be recognized by Google Maps.",
        ),
    ]
    geofence: AddressGeofence
    id: str = pydantic.Field()
    """
    ID of the Address.
    """

    latitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Latitude of the address. Will be geocoded from `formattedAddress` if not provided.
    """

    longitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Longitude of the address. Will be geocoded from `formattedAddress` if not provided.
    """

    name: str = pydantic.Field()
    """
    Name of the address.
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notes about the address.
    """

    tags: typing.Optional[typing.List[TagTinyResponse]] = pydantic.Field(default=None)
    """
    An array of all tag mini-objects that are associated with the given address entry.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ListHubPlansResponsePaginationResponseBody(UniversalBaseModel):
    """
    Pagination information
    """

    end_cursor: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="endCursor"),
        pydantic.Field(alias="endCursor", description="Cursor for the last item in the page"),
    ] = None
    has_next_page: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="hasNextPage"),
        pydantic.Field(alias="hasNextPage", description="Whether there are more pages available"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .patch_safety_events_dismissal_reason_body_request_body_code import (
    PatchSafetyEventsDismissalReasonBodyRequestBodyCode,
)


class PatchSafetyEventsDismissalReasonBodyRequestBody(UniversalBaseModel):
    """
    Dismissal reason. Required when eventState is `dismissed`. Must not be provided for other states.
    """

    code: PatchSafetyEventsDismissalReasonBodyRequestBodyCode = pydantic.Field()
    """
    The dismissal reason code associated with the event.  Valid values: `incorrect`, `minorEvent`, `other`
    """

    comment: typing.Optional[str] = pydantic.Field(default=None)
    """
    The dismissal reason comment associated with the event.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

WebhooksPostWebhooksResponseBodyEventTypesItem = typing.Union[
    typing.Literal[
        "AddressCreated",
        "AddressDeleted",
        "AddressUpdated",
        "AlertIncident",
        "AlertObjectEvent",
        "DocumentSubmitted",
        "DriverCreated",
        "DriverUpdated",
        "DvirSubmitted",
        "EngineFaultOff",
        "EngineFaultOn",
        "FormSubmitted",
        "FormUpdated",
        "GatewayUnplugged",
        "GeofenceEntry",
        "GeofenceExit",
        "IssueCreated",
        "MissingDvirPastDue",
        "PredictiveMaintenanceAlert",
        "RouteStopArrival",
        "RouteStopDeparture",
        "RouteStopEarlyLateArrival",
        "RouteStopEtaUpdated",
        "RouteStopResequence",
        "SevereSpeedingEnded",
        "SevereSpeedingStarted",
        "SpeedingEventEnded",
        "SpeedingEventStarted",
        "SuddenFuelLevelDrop",
        "SuddenFuelLevelRise",
        "VehicleCreated",
        "VehicleUpdated",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_stats_fault_codes_value_j_1939_check_engine_lights import (
    VehicleStatsFaultCodesValueJ1939CheckEngineLights,
)
from .vehicle_stats_fault_codes_value_j_1939_diagnostic_trouble_codes import (
    VehicleStatsFaultCodesValueJ1939DiagnosticTroubleCodes,
)


class VehicleStatsFaultCodesValueJ1939(UniversalBaseModel):
    """
    Vehicle fault codes for J1939 vehicles.
    """

    check_engine_lights: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueJ1939CheckEngineLights],
        FieldMetadata(alias="checkEngineLights"),
        pydantic.Field(alias="checkEngineLights"),
    ] = None
    diagnostic_trouble_codes: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsFaultCodesValueJ1939DiagnosticTroubleCodes]],
        FieldMetadata(alias="diagnosticTroubleCodes"),
        pydantic.Field(alias="diagnosticTroubleCodes", description="Diagnostic trouble codes for J1939 vehicles."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .equipment_time import EquipmentTime


class EquipmentGatewayEngineSeconds(UniversalBaseModel):
    """
    Engine seconds reading from the aux/digio cable.
    """

    time: EquipmentTime
    value: int = pydantic.Field()
    """
    The number of seconds an engine has been running as detected via engine state. Used in combination with an offset provided manually through the Samsara cloud dashboard. Useful for when assets do not report engine hours over the J1939 network. The Engine Speed SPN must be available from the ECU for this parameter to properly calculate seconds. This is supported with the following hardware configurations: AG24/AG26 + AOPEN/A9PIN/ACT9/ACT14.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .v_1_programs_for_the_camera_response_item import V1ProgramsForTheCameraResponseItem

V1ProgramsForTheCameraResponse = typing.List[V1ProgramsForTheCameraResponseItem]



# This file was auto-generated by Fern from our API Definition.

import typing

HosEldEventObjectResponseBodyMalfunctionDiagnosticCode = typing.Union[
    typing.Literal["P", "E", "T", "L", "R", "S", "O", "1", "2", "3", "4", "5", "6"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class HosShift(UniversalBaseModel):
    """
    Remaining durations and start time for the HOS on duty shift limits.
    """

    shift_remaining_duration_ms: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="shiftRemainingDurationMs"),
        pydantic.Field(
            alias="shiftRemainingDurationMs",
            description="Remaining on duty or driving time the driver in the current shift in milliseconds. For property-carrying drivers, this is the amount of time the driver can be on duty or driving before hitting the 14-hour limit.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ListUploadedMediaRequestTriggerReasonsItem = typing.Union[
    typing.Literal[
        "api",
        "panicButton",
        "periodicStill",
        "rfidEvent",
        "safetyEvent",
        "tripEndStill",
        "tripStartStill",
        "videoRetrieval",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class GoaTagTinyResponseResponseBody(UniversalBaseModel):
    """
    A minified tag object
    """

    id: str = pydantic.Field()
    """
    ID of the tag
    """

    name: str = pydantic.Field()
    """
    Name of the tag.
    """

    parent_tag_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="parentTagId"),
        pydantic.Field(
            alias="parentTagId",
            description="If this tag is part a hierarchical tag tree, this is the ID of the parent tag, otherwise this will be omitted.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .recovery_photo_response_body_status import RecoveryPhotoResponseBodyStatus


class RecoveryPhotoResponseBody(UniversalBaseModel):
    """
    A photo associated with an asset recovery event.
    """

    start_ms: int = pydantic.Field()
    """
    Timestamp of when the photo was captured, in milliseconds since epoch.
    """

    status: RecoveryPhotoResponseBodyStatus = pydantic.Field()
    """
    The availability status of the photo.  Valid values: `EXISTS`, `NOT_FOUND`, `SERVER_ERROR`
    """

    url: str = pydantic.Field()
    """
    A temporary presigned URL for the recovery photo. This URL expires at the time indicated by url_expires_at_ms.
    """

    url_expires_at_ms: int = pydantic.Field()
    """
    Timestamp when the presigned URL expires, in milliseconds since epoch.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.contact_response import ContactResponse
from ..types.list_contacts_response import ListContactsResponse
from .raw_client import AsyncRawContactsClient, RawContactsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ContactsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawContactsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawContactsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawContactsClient
        """
        return self._raw_client

    def list_contacts(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListContactsResponse:
        """
        Returns a list of all contacts in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListContactsResponse
            List of all contacts

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.contacts.list_contacts()
        """
        _response = self._raw_client.list_contacts(limit=limit, after=after, request_options=request_options)
        return _response.data

    def create_contact(
        self,
        *,
        email: typing.Optional[str] = OMIT,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContactResponse:
        """
        Add a contact to the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        email : typing.Optional[str]
            Email address of the contact.

        first_name : typing.Optional[str]
            First name of the contact.

        last_name : typing.Optional[str]
            Last name of the contact.

        phone : typing.Optional[str]
            Phone number of the contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContactResponse
            Contact was successfully added.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.contacts.create_contact()
        """
        _response = self._raw_client.create_contact(
            email=email, first_name=first_name, last_name=last_name, phone=phone, request_options=request_options
        )
        return _response.data

    def get_contact(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ContactResponse:
        """
        Get a specific contact's information.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContactResponse
            Returns the specified contact.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.contacts.get_contact(
            id="id",
        )
        """
        _response = self._raw_client.get_contact(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the given contact.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.contacts.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update_contact(
        self,
        id: str,
        *,
        email: typing.Optional[str] = OMIT,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContactResponse:
        """
        Update a specific contact's information.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the contact.

        email : typing.Optional[str]
            Email address of the contact.

        first_name : typing.Optional[str]
            First name of the contact.

        last_name : typing.Optional[str]
            Last name of the contact.

        phone : typing.Optional[str]
            Phone number of the contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContactResponse
            Updated contact object with given ID.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.contacts.update_contact(
            id="id",
        )
        """
        _response = self._raw_client.update_contact(
            id, email=email, first_name=first_name, last_name=last_name, phone=phone, request_options=request_options
        )
        return _response.data


class AsyncContactsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawContactsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawContactsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawContactsClient
        """
        return self._raw_client

    async def list_contacts(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListContactsResponse:
        """
        Returns a list of all contacts in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListContactsResponse
            List of all contacts

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.contacts.list_contacts()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_contacts(limit=limit, after=after, request_options=request_options)
        return _response.data

    async def create_contact(
        self,
        *,
        email: typing.Optional[str] = OMIT,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContactResponse:
        """
        Add a contact to the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        email : typing.Optional[str]
            Email address of the contact.

        first_name : typing.Optional[str]
            First name of the contact.

        last_name : typing.Optional[str]
            Last name of the contact.

        phone : typing.Optional[str]
            Phone number of the contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContactResponse
            Contact was successfully added.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.contacts.create_contact()


        asyncio.run(main())
        """
        _response = await self._raw_client.create_contact(
            email=email, first_name=first_name, last_name=last_name, phone=phone, request_options=request_options
        )
        return _response.data

    async def get_contact(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ContactResponse:
        """
        Get a specific contact's information.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContactResponse
            Returns the specified contact.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.contacts.get_contact(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_contact(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the given contact.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.contacts.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update_contact(
        self,
        id: str,
        *,
        email: typing.Optional[str] = OMIT,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContactResponse:
        """
        Update a specific contact's information.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Alert Contacts** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the contact.

        email : typing.Optional[str]
            Email address of the contact.

        first_name : typing.Optional[str]
            First name of the contact.

        last_name : typing.Optional[str]
            Last name of the contact.

        phone : typing.Optional[str]
            Phone number of the contact.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContactResponse
            Updated contact object with given ID.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.contacts.update_contact(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_contact(
            id, email=email, first_name=first_name, last_name=last_name, phone=phone, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .safety_event_v_2_behavior_labels_response_body_label import SafetyEventV2BehaviorLabelsResponseBodyLabel
from .safety_event_v_2_behavior_labels_response_body_source import SafetyEventV2BehaviorLabelsResponseBodySource


class SafetyEventV2BehaviorLabelsResponseBody(UniversalBaseModel):
    """
    The most up-to-date behavior labels associated with the safety event. These labels can be updated by Safety Admins.
    """

    label: typing.Optional[SafetyEventV2BehaviorLabelsResponseBodyLabel] = pydantic.Field(default=None)
    """
    The label associated with the safety event. Labels can be updated by Safety Admins, if a label is removed from a safety event it will be removed from this list.  Valid values: `Acceleration`, `AggressiveDriving`, `BluetoothHeadset`, `Braking`, `ContextConstructionOrWorkZone`, `ContextSnowyOrIcy`, `ContextVulnerableRoadUser`, `ContextWet`, `Crash`, `DefensiveDriving`, `DidNotYield`, `Drinking`, `Drowsy`, `Eating`, `EatingDrinking`, `EdgeDistractedDriving`, `EdgeRailroadCrossingViolation`, `FollowingDistance`, `FollowingDistanceModerate`, `FollowingDistanceSevere`, `ForwardCollisionWarning`, `GenericDistraction`, `GenericTailgating`, `HarshTurn`, `HeavySpeeding`, `HighSpeedSuddenDisconnect`, `HosViolation`, `Idling`, `Invalid`, `LaneDeparture`, `LateResponse`, `LeftTurn`, `LightSpeeding`, `MaxSpeed`, `MobileUsage`, `ModerateSpeeding`, `NearCollison`, `NearPedestrianCollision`, `NoSeatbelt`, `ObstructedCamera`, `OperationalEvent`, `OtherViolation`, `Passenger`, `PolicyViolationMask`, `ProtectiveEquipment`, `RanRedLight`, `RearCollisionWarning`, `Reversing`, `RollingStop`, `RolloverProtection`, `SevereSpeeding`, `Smoking`, `Speeding`, `UTurn`, `UnsafeManeuver`, `UnsafeParking`, `VehicleInBlindSpotWarning`, `VulnerableRoadUserCollisionWarning`, `YawControl`
    """

    source: typing.Optional[SafetyEventV2BehaviorLabelsResponseBodySource] = pydantic.Field(default=None)
    """
    The source of the label associated with the safety event.  Valid values: `automated`, `userGenerated`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .order_custom_property_response_body import OrderCustomPropertyResponseBody
from .order_quantity_response_body import OrderQuantityResponseBody
from .order_task_response_body import OrderTaskResponseBody


class OrderObjectResponseBody(UniversalBaseModel):
    """
    Order object
    """

    created_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="The timestamp (in UTC) when the order was created"),
    ]
    custom_properties: typing_extensions.Annotated[
        typing.List[OrderCustomPropertyResponseBody],
        FieldMetadata(alias="customProperties"),
        pydantic.Field(alias="customProperties", description="An array of custom property values for the order"),
    ]
    customer_order_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="customerOrderId"),
        pydantic.Field(alias="customerOrderId", description="The customer-provided identifier for the order"),
    ]
    delivery: typing.Optional[OrderTaskResponseBody] = None
    hub_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="hubId"),
        pydantic.Field(alias="hubId", description="The ID of the hub the order belongs to"),
    ]
    id: str = pydantic.Field()
    """
    The Samsara-generated unique identifier (UUID) for the order
    """

    pickup: typing.Optional[OrderTaskResponseBody] = None
    plan_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="planId"),
        pydantic.Field(alias="planId", description="The ID of the plan the order belongs to"),
    ]
    priority: int = pydantic.Field()
    """
    Priority of the order (e.g., 1 for high, 5 for low)
    """

    quantities: typing.List[OrderQuantityResponseBody] = pydantic.Field()
    """
    An array of quantities for the order
    """

    skills_required: typing_extensions.Annotated[
        typing.List[str],
        FieldMetadata(alias="skillsRequired"),
        pydantic.Field(alias="skillsRequired", description="An array of skill IDs required to fulfill the order"),
    ]
    updated_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAt"),
        pydantic.Field(alias="updatedAt", description="The timestamp (in UTC) when the order was last updated"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

VehicleSensorConfigurationAreaPosition = typing.Union[
    typing.Literal["Position_Front", "Position_Middle", "Position_Back"], typing.Any
]
